import os

PROVIDER = os.environ.get('CLOUD_SERVICES_PROVIDER')
REGION = os.environ.get('CLOUD_SERVICES_REGION')

# Notification Bus services
NOTIFICATION = 'Notification'
SUSCRIPTION = 'Subscription'



import json
import logging
import requests
import boto3
from flask import request
from .. import config
from ..constants import responses as rsp
from ..config import NOTIFICATION, SUSCRIPTION
from ..contract import BusNotificationsContract
from ..contract import DataListener


__all__ = (
    'BusNotifications',
)


class BusNotifications(BusNotificationsContract):

    def listener(self) -> DataListener:
        """Endpoint of Suscription and Reception of Notifications of AWS sns.
        Implementation of `listener` of AWS notifications bus
        for flask.

        Returns:
            ..contract.DataListener:
        """
        try:
            data = json.loads(request.data)
        except Exception as e:
            logging.error(str(e))
            return DataListener(
                response=(rsp.AWS_BAD_REQUEST, 400)
            )
        message_type = request.headers.get('X-Amz-Sns-Message-Type')
        topic_arn = request.headers.get('X-Amz-Sns-Topic-Arn')
        if message_type == 'Notification':
            # Receive notification from AWS channel and call decorated function
            result = DataListener(
                message_type=NOTIFICATION,
                message=data['Message'],
                topic_arn=topic_arn,
                response=rsp.AWS_OK,
            )
        elif (message_type == 'SubscriptionConfirmation'
                and 'SubscribeURL' in data):
            # Confirm suscription of AWS channel
            requests.get(data['SubscribeURL'])
            result = DataListener(
                message_type=SUSCRIPTION,
                topic_arn=topic_arn,
                response=rsp.AWS_OK
            )
        else:
            result = DataListener(
                response=(rsp.AWS_BAD_REQUEST, 400)
            )
        return result

    def publisher(
        self,
        topic_arn: str = None,
        message: str = None
    ) -> dict:
        """Method to publish in sns of AWS.
        Implementation of `publisher` to AWS notifications bus
        for Flask.

        Args:
            topic_arn (str)
            message (str)

        Returns:
            dict: Response returned by client sns
        """
        client = boto3.client('sns', region_name=config.REGION)
        # Publish a simple message to the specified SNS topic
        response = client.publish(
            TopicArn=topic_arn,
            Message=message
        )

        return response



from .services.notifications import listener as notifications_listener
from .services.notifications import publisher as notifications_publisher

__all__ = (
    'notifications_listener',
    'notifications_publisher',
)



import abc


class DataListener:
    """Object used for BusNotifications.listener()

    Attributes:
        message: type unknown
        response: type unknown
    """

    def __init__(
        self,
        message_type: str = None,
        topic_arn: str = None,
        message=None,
        response=None
    ):
        self.message_type = message_type
        self.topic_arn = topic_arn
        self.message = message
        self.response = response


class BusNotificationsContract(metaclass=abc.ABCMeta):
    """Contract for notifications buses"""

    @abc.abstractmethod
    def listener(self) -> DataListener:
        """It must Suscribe or Receive Notifications from Notifications Bus
        Implementation of `listener` of Notifications Bus

        Returns:
            DataListener:
        """
        pass

    @abc.abstractmethod
    def publisher(
        self,
        topic_arn: str,
        message,
        *args,
        **kwargs
    ):
        """It must Publish to Notifications Bus

        Args:
            topic_arn (str): topic arn of channel
            message (str): message to publish

        Note: Any parameter the adapter needs,
            wich does not deliver in request
        """
        pass
